/*
	'C' language interface to Argus SDK
	V3.0 and 3.1

	(c) 2001 Iceni Technology Ltd.
*/
#include "ipAbstract.h"
#include "icnFeature.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifdef WIN
#	define VBCALL	__stdcall
#else
#	define VBCALL	
#endif

#define kMaxVersion	200	/* Min. length of version string buffer */

#define kTableUserAnnot	1
#define kImageUserAnnot	2
#define kCropUserAnnot	3

#ifndef _ICE_DEFS
	typedef int ICError;
#	define kNoError	0
#	define u8 unsigned char
	typedef unsigned short ichar;		/* Iceni char */
#endif

#ifndef _RUNWAY_DEFS
	typedef struct ABSTRACT_RUNWAY* ICNRunway;
	typedef struct ABSTRACT_PAGE* ICNPage;
	typedef struct ABSTRACT_DOC* ICNDoc;
	typedef struct ABSTRACT_STYLE* ICNStyle;
	typedef void (*ICNProgMsg)(void* guiData, int level, char* text);
	typedef void (*ICNProgGauge)(void* guiData, int percent);
	typedef int (*ICNProgCancel)(void* guiData);
#endif

/* ---- SDK 3.0 ------------------------------------------------------------- */

/* Start the engine */
void* icnArgusCreate();

/* Load the config File */
ICError icnArgusLoadConfigs(void* vargus, char* configFile);

/* Init the progress callbacks and a ptr to private data of your own */
void icnArgusSetProgress(void* va, ICNProgMsg msg, ICNProgGauge gauge, 
								 ICNProgCancel cancel, void* guiData);

/* Get/Set document features. Use to overrride features set by config files
	since you cannot use VALUE macros to influence features. */
unsigned int icnArgusGetDocFeatures(void * vargus);
void icnArgusSetDocFeatures(void* vargus, unsigned int features);

/* This may be called numerous times each with numerous files */
ICError icnArgusExtract(void* va, char** files, int nofFiles, 
								  int startPage, int endPage);

/* New for sdk 4 - output a new config containing format entries for
	all entities found in a doc's structure tree. (Tagged pdf only) */
ICError icnArgusScanEntities(void* vargus, char* fname, char* configName);

/* Sets the location of the license file. Default is 'argus.key' */
ICError icnArgusSetKeyFile(void* va, char* keyFName);

/* Set the actual key code used for checking */
void icnArgusSetKey(void* va, char* key);

/* Adding user-specified annotations */
ICError icnArgusAddUserAnnot(void* va, u8 type, char* rangeStr, IPRect* bbox);

/* Add your own macro keys for the VALUE macro. Useful for customising 
	configuration files on the fly. */
ICError icnArgusValueAdd(void* va, char* key, char* value);

/* Stop the engine */
void icnArgusDestroy(void* va);

/* VALUE macro - Adding Key,value pairs. For customising config files. */
ICError icnValueMacroAdd(ICNRunway runway, char* key, char* value);
ICError icnValueMacroAddiStr(ICNRunway runway, ichar* key, ichar* ivalue);

/* ---- New / Modified for Argus SDK 3.2 ------------------------------------ */

/* Get version message. Passes back nofBytes written to buffer. Buffer 
	should be at least kMaxVersion bytes long. */
int icnArgusGetVersion(char* msg);

/* Load a specific document into Argus. 
	Should be called after config has been loaded. */
ICNDoc icnArgusDocCreate(void* va, char* fname);

/* Create, interrogate, destroy a page. */
ICNPage icnDocGetPage(ICNDoc icnDoc, int pageNum);
int icnDocGetPageCount(ICNDoc icnDoc);
ichar* icnPageParseText(ICNPage icnPage, ICNStyle, ichar*, int* nofBytes);
ICNStyle icnArgusGetStyle(void* vargus);
void icnPageDestroy(ICNPage);

/* Analyse the font usage accross the pages you want to extract later */
ICError icnArgusDocAnalyse(void* va, ICNDoc, int start, int end, float ticks);

/* Extract the text from the given page. Returns a buffer which must be freed */
char* icnArgusDocGetText(void* va, ICNDoc, int pageNum, int* nofBytes);
ichar* icnArgusDocGetiText(void* va, ICNDoc, int pageNum, int* nofChars);

/* Just like icnArgusExtract but using a single doc */
ICError icnArgusDocExtract(void* va, ICNDoc icnDoc, int start, int end);

/* Given some text possibly containing macros, expand them in the context of
	the given document and return the resulting string. */
char* icnArgusDocParseText(void* va, ICNDoc doc, char* text, int* nofBytes);

/* Returns the bookmark dump from the document */
ichar* icnArgusDocGetBookmarks(void* va, ICNDoc doc);

/* Close the open document - freeing up any associated memory */
void icnDocDestroy(ICNDoc);

/* Use this to free text buffers (or any kind of allocated buffer). Do not
	use free() since this cannot free DLL allocated memory in some cases. */
void icnDestroyText(char*);


#ifndef _RUNWAY_DEFS
/* ---- New for Argus 5.1 --------------------------------------------------- */
/* Set and retreive the feature bit flags for the document. 
	Simmilar to icnArgusGetDocFeatures() but can be applied on a per document
	basis. */
void icnDocSetFeatures(ICNDoc icnDoc, unsigned int features);
unsigned int icnDocGetFeatures(ICNDoc icnDoc);
#endif

/* ---- Access to underlying PDF interpreter -------------------------------- */
void ipSetUserPassword(char* password, int nofBytes);

/* ---- Simple Page Rendering - SDK 3.0 ------------------------------------- */

typedef struct 
{
	float x;
	float y;
} ICNPoint;

typedef struct
{
	ICNPoint bl;    /* bottom-left  */
	ICNPoint tl;    /* top-left     */
	ICNPoint tr;    /* top-right    */
	ICNPoint br;    /* bottom-right */
} ICNQuad;

typedef struct
{
	float red;
	float green;
	float blue;
} ICNColour;

typedef int (*ICNPolyFunc)(IPPoint*, int nofPoints, ICNColour* rgb, void* gui);
typedef int (*ICNImageFunc)(IPQuad*, IPVImage, int width, int height, int bpc, 
										int nofChannels, void* gui);

typedef struct
{
	int (*beginPage)(IPRect* media, IPRect* crop, void* gui);
	int (*showText)(IPQuad* q, const ichar* text, int nofChars, 
						 u8* fname, void* gui);
	ICNImageFunc paintImage;
	ICNPolyFunc drawPoly;
	ICNPolyFunc fillPoly;
	int (*endPage)(void* gui);
	void* dataBlock;
} ICNArgusRenderCallbacks;

void* icnArgusRenderCreate(char* fname, ICNArgusRenderCallbacks* mon);
int icnArgusRenderGetPageCount(void* vr);
ICError icnArgusRenderPage(void* vr,  int pageNum);
void icnArgusRenderDestroy(void* vr);

IPVStream icnArgusImageBegin(IPVImage);
int icnArgusImageRead(IPVStream vs, u8* buff, int nofBytes);
void icnArgusImageEnd(IPVStream vs);

/* ---- Simple Error Handling - SDK 3.0 ------------------------------------- */

/* Returns text of error messages */
char* icnErrorGetText(void);

/* Returns standard description of error code */
const char* icnErrorGetStdText(void);

int icnErrorGetCode(void);

#ifdef __cplusplus
}
#endif
